﻿using EonaCat.SecretVault.Services;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Mvc;
using System.Threading.Tasks;

// This file is part of the EonaCat project(s) which is released under the Apache License.
// See the LICENSE file or go to https://EonaCat.com/License for full license details.

namespace EonaCat.SecretVault.Controllers
{
    [ApiController]
    [Route("api/[controller]")]
    public class SecretsController : ControllerBase
    {
        private readonly SecretService _svc;
        public SecretsController(SecretService svc) => _svc = svc;

        [HttpPost("store")]
        public async Task<IActionResult> Store([FromQuery] string key, [FromBody] string secret)
        {
            var user = HttpContext.Items["User"]?.ToString() ?? "unknown";
            var tenant = HttpContext.Items["TenantId"]?.ToString() ?? "default";
            await _svc.StoreAsync(tenant, key, secret, user);
            return Ok();
        }

        [HttpDelete("delete")]
        public async Task<IActionResult> Delete([FromQuery] string key)
        {
            var user = HttpContext.Items["User"]?.ToString() ?? "unknown";
            var tenant = HttpContext.Items["TenantId"]?.ToString() ?? "default";
            var deleted = await _svc.DeleteAsync(tenant, key, user);
            if (!deleted)
            {
                return NotFound();
            }

            return Ok();
        }

        [HttpGet("retrieve")]
        public IActionResult Retrieve([FromQuery] string key)
        {
            var user = HttpContext.Items["User"]?.ToString() ?? "unknown";
            var tenant = HttpContext.Items["TenantId"]?.ToString() ?? "default";
            var secret = _svc.Retrieve(tenant, key, user);
            if (secret == null)
            {
                return NotFound();
            }

            return Ok(secret);
        }
    }
}
