﻿using EonaCat.SecretVault.Data;
using Microsoft.AspNetCore.Http;
using Microsoft.Extensions.DependencyInjection;
using System;
using System.Linq;
using System.Security.Cryptography;
using System.Text;
using System.Threading.Tasks;

// This file is part of the EonaCat project(s) which is released under the Apache License.
// See the LICENSE file or go to https://EonaCat.com/License for full license details.

namespace EonaCat.SecretVault.Middleware
{
    public class ApiKeyMiddleware
    {
        private readonly RequestDelegate _next;
        private const string ApiKeyHeaderName = "X-Api-Key";

        public ApiKeyMiddleware(RequestDelegate next) => _next = next;

        public async Task InvokeAsync(HttpContext context)
        {
            if (!context.Request.Headers.TryGetValue(ApiKeyHeaderName, out var extractedApiKey))
            {
                context.Response.StatusCode = 401;
                await context.Response.WriteAsync("API Key is missing");
                return;
            }

            var db = context.RequestServices.GetRequiredService<SecretDbContext>();
            var tenant = db.ApiKeys.AsEnumerable()
                .FirstOrDefault(k => VerifyApiKey(extractedApiKey, k.ApiKeyHash));

            if (tenant == null)
            {
                context.Response.StatusCode = 403;
                await context.Response.WriteAsync("Unauthorized API Key");
                return;
            }

            context.Items["TenantId"] = tenant.TenantId;
            context.Items["User"] = "apikey-user";
            context.Items["Scope"] = tenant.Scope;
            await _next(context);
        }

        private bool VerifyApiKey(string apiKey, string storedHash)
        {
            using var sha256 = SHA256.Create();
            var hash = sha256.ComputeHash(Encoding.UTF8.GetBytes(apiKey));
            var hashBase64 = Convert.ToBase64String(hash);
            return hashBase64 == storedHash;
        }
    }

}
