﻿using EonaCat.SecretVault.Models;
using Microsoft.AspNetCore.Mvc.Filters;
using System;

// This file is part of the EonaCat project(s) which is released under the Apache License.
// See the LICENSE file or go to https://EonaCat.com/License for full license details.

namespace EonaCat.SecretVault.Middleware
{
    [AttributeUsage(AttributeTargets.Method)]
    public class RequireRoleAttribute : Attribute, IAuthorizationFilter
    {
        private readonly ApiRole _requiredRole;

        public RequireRoleAttribute(ApiRole requiredRole)
        {
            _requiredRole = requiredRole;
        }

        public void OnAuthorization(AuthorizationFilterContext context)
        {
            if (!context.HttpContext.Items.TryGetValue("Role", out var roleObj))
            {
                return;
            }

            if (Enum.TryParse(roleObj?.ToString(), out ApiRole actualRole))
            {
                if (actualRole >= _requiredRole)
                {
                    return;
                }
            }

            context.Result = new Microsoft.AspNetCore.Mvc.ForbidResult();
        }
    }

}
